#include <stdio.h>
#include <M5Stack.h>
extern "C" {
  #include "bmp.h"
  #include "numap.h"
}

#define KEYBOARD_I2C_ADDR 0X08
#define KEYBOARD_INT      5

#define MODE_TITLE    0
#define MODE_GAME     1
#define MODE_ENDING   2
#define MODE_GAMEOVER 3

#define B_UP      0xFE
#define B_LEFT    0xFB
#define B_DOWN    0xFD
#define B_RIGHT   0xF7
#define B_SELECT  0xBF
#define B_START   0x7F
#define B_A       0xEF
#define B_B       0xDF
#define B_AB      0xCF

#define B_END     0xFF

int mode = MODE_TITLE;
int keyDown = 0;

void setup() {
  
  randomSeed(analogRead(0));
  
  M5.begin();
  Wire.begin();

  M5.Lcd.clear(BLACK);

}

void loop() {

  if (mode == MODE_TITLE) {
    modeTitle();
  } else if (mode == MODE_GAME) {
    modeGame();
  } else if (mode == MODE_ENDING) {
    modeEnding();
  } else if (mode == MODE_GAMEOVER) {
    modeGameover();
  }
  
  M5.update();
  
}

void modeTitle(void) {
  M5.Lcd.setTextColor(YELLOW);
  M5.Lcd.setTextSize(4);
  M5.Lcd.setCursor(10, 20);
  M5.Lcd.println("NU PICK UP");
  M5.Lcd.setCursor(45, 70);
  M5.Lcd.println("GOLD INGOTS");
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(45, 150);
  M5.Lcd.println("PUSH [START] BUTTON");
  M5.Lcd.setTextColor(CYAN);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(92, 205);
  M5.Lcd.println("2020 UO Soft");
  
  uint8_t keyVal = getKey();
  if (keyVal == B_START) {
    M5.Lcd.clear(BLACK);
    numap_create();
    printGold();
    mode = MODE_GAME;
  }
}

void modeGame(void) {
  int sizeX = 14;
  int sizeY = 14;

  int offsetX = 0;
  int offsetY = 0;

  int posX;
  int posY;

  for (int y = 0; y < 17; y++) {
    for (int x = 0; x < 17; x++) {
      posX = x * sizeX + offsetX;
      posY = y * sizeY + offsetY;
      if (!(x == numap_nux() && y == numap_nuy())) {
        if (StageMap[y][x] == 0) {
          M5.Lcd.fillRect(posX, posY, sizeX, sizeY, BLACK);
        }
        if (StageMap[y][x] == 1) {
          M5.Lcd.drawBitmap(posX, posY, sizeX, sizeY, (uint16_t *)imgWall);
        }
        if (StageMap[y][x] == 2) {
          M5.Lcd.drawBitmap(posX, posY, sizeX, sizeY, (uint16_t *)imgGold);
        }
      }
    }
  }
  
  posX = numap_nux() * sizeX + offsetX;
  posY = numap_nuy() * sizeY + offsetY;
  M5.Lcd.drawBitmap(posX, posY, sizeX, sizeY, (uint16_t *)imgNu);

  M5.Lcd.setTextColor(RED);
  M5.Lcd.setTextSize(1);
  M5.Lcd.setCursor(260, 185);
  M5.Lcd.println("[A]+[B]");
  M5.Lcd.setCursor(260, 195);
  M5.Lcd.println("GIVE UP");

  if (Gold == 0) {
    M5.Lcd.clear(BLACK);
    mode = MODE_ENDING; 
  }

  int backupGold = Gold;

  uint8_t keyVal = getKey();
  if (keyVal == B_UP) {
    numap_movenu(0, -1);
  } else if (keyVal == B_LEFT) {
    numap_movenu(-1, 0);
  } else if (keyVal == B_DOWN) {
    numap_movenu(0, 1);
  } else if (keyVal == B_RIGHT) {
    numap_movenu(1, 0);
  } else if (keyVal == B_AB) {
    M5.Lcd.clear(BLACK);
    mode = MODE_GAMEOVER; 
  }

  if (backupGold != Gold) {
    printGold();
  }

}

void modeEnding(void) {
  M5.Lcd.setTextColor(YELLOW);
  M5.Lcd.setTextSize(3);
  M5.Lcd.setCursor(17, 60);
  M5.Lcd.println("CONGRATULATIONS!");
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(70, 170);
  M5.Lcd.println("PUSH [A] BUTTON");
  
  uint8_t keyVal = getKey();
  if (keyVal == B_A) {
    M5.Lcd.clear(BLACK);
    mode = MODE_TITLE;
  }
}

void modeGameover(void) {
  M5.Lcd.setTextColor(RED);
  M5.Lcd.setTextSize(4);
  M5.Lcd.setCursor(52, 60);
  M5.Lcd.println("GAME OVER");
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(70, 170);
  M5.Lcd.println("PUSH [A] BUTTON");
  
  uint8_t keyVal = getKey();
  if (keyVal == B_A) {
    M5.Lcd.clear(BLACK);
    mode = MODE_TITLE;
  }
}

void printGold(void) {
  M5.Lcd.setTextColor(CYAN);
  M5.Lcd.setTextSize(2);
  M5.Lcd.setCursor(255, 40);
  M5.Lcd.println("GOLD");
  M5.Lcd.fillRect(250, 62, 60, 20, BLACK);
  M5.Lcd.setCursor(264, 62);
  M5.Lcd.printf("%03d", Gold);
  M5.Lcd.drawLine(250, 82, 310, 82, CYAN);
  M5.Lcd.setCursor(264, 89);
  M5.Lcd.printf("%03d", MaxGold);
}

uint8_t getKey(void) {
  uint8_t keyVal = 0;
  if(digitalRead(KEYBOARD_INT) == LOW) {
    Wire.requestFrom(KEYBOARD_I2C_ADDR, 1);  // request 1 byte from keyboard
    if (Wire.available()) { 
      keyVal = Wire.read();                  // receive a byte as character
    }
  }
  if (keyVal == B_END) {
    keyDown = 0;
    keyVal = 0;
  } else if (keyDown == 1) {
    keyVal = 0;
  } else if (keyVal != 0) {
    keyDown = 1;
  }
  return keyVal;
}
